@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -9,7 +9,15 @@
 LICENSE END */
 /*!
     This crate contains the internal procedural macros
-    used by the sixtyfps corelib crate
+    used by the sixtyfps-corelib crate
+
+**NOTE**: This library is an **internal** crate for the [SixtyFPS project](https://sixtyfps.io).
+This crate should **not be used directly** by applications using SixtyFPS.
+You should use the `sixtyfps` crate instead.
+
+**WARNING**: This crate does not follow the semver convention for versioning and can
+only be used with `version = "=x.y.z"` in Cargo.toml.
+
 */
 
 extern crate proc_macro;
@@ -39,6 +47,7 @@ pub fn sixtyfps_element(input: TokenStream) -> TokenStream {
     };
 
     let mut pub_prop_field_names = Vec::new();
+    let mut pub_prop_field_names_normalized = Vec::new();
     let mut pub_prop_field_types = Vec::new();
     let mut property_names = Vec::new();
     let mut property_visibility = Vec::new();
@@ -47,6 +56,7 @@ pub fn sixtyfps_element(input: TokenStream) -> TokenStream {
         if let Some(property_type) = property_type(&field.ty) {
             let name = field.ident.as_ref().unwrap();
             if matches!(field.vis, syn::Visibility::Public(_)) {
+                pub_prop_field_names_normalized.push(normalize_identifier(name));
                 pub_prop_field_names.push(name);
                 pub_prop_field_types.push(&field.ty);
             }
@@ -59,32 +69,32 @@ pub fn sixtyfps_element(input: TokenStream) -> TokenStream {
     let (plain_field_names, plain_field_types): (Vec<_>, Vec<_>) = fields
         .iter()
         .filter(|f| {
-            f.attrs
-                .iter()
-                .find(|attr| {
-                    attr.parse_meta()
-                        .ok()
-                        .map(|meta| match meta {
-                            syn::Meta::Path(path) => path
-                                .get_ident()
-                                .map(|ident| ident.to_string() == "rtti_field")
-                                .unwrap_or(false),
-                            _ => false,
-                        })
-                        .unwrap_or(false)
-                })
-                .is_some()
+            f.attrs.iter().any(|attr| {
+                attr.parse_meta()
+                    .ok()
+                    .map(|meta| match meta {
+                        syn::Meta::Path(path) => {
+                            path.get_ident().map(|ident| *ident == "rtti_field").unwrap_or(false)
+                        }
+                        _ => false,
+                    })
+                    .unwrap_or(false)
+            })
         })
         .map(|f| (f.ident.as_ref().unwrap(), &f.ty))
         .unzip();
+    let plain_field_names_normalized =
+        plain_field_names.iter().map(|f| normalize_identifier(*f)).collect::<Vec<_>>();
 
     let mut callback_field_names = Vec::new();
+    let mut callback_field_names_normalized = Vec::new();
     let mut callback_args = Vec::new();
     let mut callback_rets = Vec::new();
     for field in fields {
         if let Some((arg, ret)) = callback_arg(&field.ty) {
             if matches!(field.vis, syn::Visibility::Public(_)) {
                 let name = field.ident.as_ref().unwrap();
+                callback_field_names_normalized.push(normalize_identifier(name));
                 callback_field_names.push(name);
                 callback_args.push(arg);
                 callback_rets.push(ret);
@@ -95,6 +105,7 @@ pub fn sixtyfps_element(input: TokenStream) -> TokenStream {
     let item_name = &input.ident;
 
     quote!(
+        #[allow(clippy::nonstandard_macro_braces)]
         #[cfg(feature = "rtti")]
         impl BuiltinItem for #item_name {
             fn name() -> &'static str {
@@ -104,21 +115,21 @@ pub fn sixtyfps_element(input: TokenStream) -> TokenStream {
                 vec![#( {
                     const O : MaybeAnimatedPropertyInfoWrapper<#item_name, #pub_prop_field_types> =
                         MaybeAnimatedPropertyInfoWrapper(#item_name::FIELD_OFFSETS.#pub_prop_field_names);
-                    (stringify!(#pub_prop_field_names), (&O).as_property_info())
+                    (#pub_prop_field_names_normalized, (&O).as_property_info())
                 } ),*]
             }
             fn fields<Value: ValueType>() -> Vec<(&'static str, &'static dyn FieldInfo<Self, Value>)> {
                 vec![#( {
                     const O : const_field_offset::FieldOffset<#item_name, #plain_field_types, const_field_offset::AllowPin> =
                         #item_name::FIELD_OFFSETS.#plain_field_names;
-                    (stringify!(#plain_field_names), &O as &'static dyn FieldInfo<Self, Value>)
+                    (#plain_field_names_normalized, &O as &'static dyn FieldInfo<Self, Value>)
                 } ),*]
             }
             fn callbacks<Value: ValueType>() -> Vec<(&'static str, &'static dyn CallbackInfo<Self, Value>)> {
                 vec![#( {
                     const O : const_field_offset::FieldOffset<#item_name, Callback<#callback_args, #callback_rets>, const_field_offset::AllowPin> =
                          #item_name::FIELD_OFFSETS.#callback_field_names;
-                    (stringify!(#callback_field_names), &O as  &'static dyn CallbackInfo<Self, Value>)
+                    (#callback_field_names_normalized, &O as  &'static dyn CallbackInfo<Self, Value>)
                 } ),*]
             }
         }
@@ -134,6 +145,10 @@ pub fn sixtyfps_element(input: TokenStream) -> TokenStream {
     .into()
 }
 
+fn normalize_identifier(name: &syn::Ident) -> String {
+    name.to_string().replace('_', "-")
+}
+
 // Try to match `Property<Foo>` on the syn tree and return Foo if found
 fn property_type(ty: &syn::Type) -> Option<&syn::Type> {
     if let syn::Type::Path(syn::TypePath { path: syn::Path { segments, .. }, .. }) = ty {
@@ -144,9 +159,7 @@ fn property_type(ty: &syn::Type) -> Option<&syn::Type> {
         }) = segments.first()
         {
             match args.first() {
-                Some(syn::GenericArgument::Type(property_type))
-                    if ident.to_string() == "Property" =>
-                {
+                Some(syn::GenericArgument::Type(property_type)) if *ident == "Property" => {
                     return Some(property_type)
                 }
                 _ => {}
